# coding=UTF-8
import json
from collections import OrderedDict


#请修改这个函数的代码修改配置选项，#revise the following code paras to gen the config json file
def gen_config():

    json_data = OrderedDict()

    #这里修改云模式，公有云为True，私有云为False
    public_cloud = False  # set the cloud mode

    if(public_cloud):
        json_data["cloud_mode"]="public"
        #填写下面公有云的配置项
        json_data["projectid"]=""                  #公有云项目ID
        json_data["device_id"]=""                  #公有云设备ID
        json_data["device_name"]=""
        json_data["password"]=""                   #公有云设备Token
        json_data["certificate"]="./device.pem"    #公有云证书
        json_data["sdk_mode"]="server2"            
    else:
        json_data["cloud_mode"]="private"
        #填写下面私有化的配置项
        json_data["device_id"]=""                  #设备ID   
        json_data["device_name"]=""
        json_data["server_ip"]=""                  #接入服务IP
        json_data["server_port"]=2883              #接入服务端口
        json_data["rtc_server_ip"]=""              #rtc服务IP
        json_data["rtc_server_port"]=3000          #rtc服务端口
        json_data["sdk_mode"]="server"             
    

    #如果有本地指定端口要求（如需要配置端口段映射），请修改这里
    port_conf = False #set the port or not
    if(port_conf):
        json_data["max_port"]=50000
        json_data["min_port"]=50100

    #如果有指定网卡需求或多网传输需求，请修改这里
    netbind_conf = False #bind target network interface or not
    if(netbind_conf):
        json_data["network_bind"]=["xx.xx.xx.xx","xx.xx.xx.xx"]  # 指定网卡的IP， target network interface ip; 多网传输绑定两个IP
    
    #如果有音频需求，请修改这里
    audio_conf = False #open audio or not
    if(audio_conf):
        json_data["audio_enable"]=1   #打开声音采集
        json_data["audio_receive"]=1  #打开声音接收
    else:
        json_data["audio_enable"]=0
        json_data["audio_receive"]=0

    #如果有日志需求，请修改这里
    json_data["log_enable"]=1  #open log， 1 正常日志模式  2 调试日志模式


    #--------下面是设置视频流的配置-----------

    #修改设置视频流的路数，数量需要video_arry的大小一致
    video_num = 2  #video stream num, keep same size as video_arry
    json_data["device_streams"]=video_num
    video_arry=[]

    #请按顺序，在下面添加视频流项目，add video stream item
    video_arry.append(v4l2_video_item(30,1920,1080,1920,1080,2000,1000,0))         #举例添加一路v4l2相机(GMSL/USB等）视频
    video_arry.append(rtsp_enc_video_item(25,1920,1080,"rtsp://admin:admin@192.168.1.1/"))    #举例添加一路网络摄像头视频

    #这个是v4l2视频流添加方式， video_arry.append(v4l2_video_item(30,1920,1080,1920,1080,2000,1000,0))
    #这个是外部输入图像添加方式，video_arry.append(outside_video_item(30,1920,1080,1920,1080,2000,1000))
    #这个是外部输入编码流添加方式，video_arry.append(outenc_video_item())
    #这个是网络相机编码透传视频流的添加方式，video_arry.append(rtsp_enc_video_item(25,1920,1080,"rtsp://admin:admin@192.168.1.1/"))
    #这个是网络相机转码传输视频流的添加方式，video_arry.append(normal_url_video_item(25,1920,1080,1920,1080,2000,1000,"rtsp://admin:admin@192.168.1.1/"))
    json_data["streams_config"]=video_arry
    
    if(len(video_arry) < video_num):
        print("video_arry size is small than video_num\n")
        return

    with open('config.json','w') as f1:
        json.dump(json_data,f1,indent=2)



#v4l2视频流配置生成，v4l2 capture video stream
def v4l2_video_item(fps, width, height, encode_width, encode_height, bps, min_bps, camera_index, codec=0):
    video_item=OrderedDict()
    video_item["fps"]=fps          #视频帧率
    video_item["width"]=width      #视频采集/输入宽
    video_item["height"]=height    #视频采集/输入高
    video_item["encode_width"]=encode_width     #视频传输宽
    video_item["encode_height"]=encode_height   #视频传输高
    video_item["bps"]=bps                       #视频传输期望码率，影响正常情况清晰度，建议1080P 2500， 720P 2000； H265可再降低500
    video_item["min_bps"]=min_bps               #视频传输弱网码率，弱网下能接受的最小码率，建议1080P 1500， 720P 1000； H265可再降低300
    video_item["codec"]=codec    #编码方式 0 H264  1 H265
    video_item["protocol"]="v4l2"
    video_item["camera"]=camera_index
    return video_item

#外部输入图像视频流配置生成，external YUV video input video stream
def outside_video_item(fps,width, height, encode_width, encode_height, bps, min_bps, codec=0):
    video_item=OrderedDict()
    video_item["fps"]=fps          #视频帧率
    video_item["width"]=width      #视频采集/输入宽
    video_item["height"]=height    #视频采集/输入高
    video_item["encode_width"]=encode_width     #视频传输宽
    video_item["encode_height"]=encode_height   #视频传输高
    video_item["bps"]=bps                       #视频传输期望码率，影响正常情况清晰度，建议1080P 2500， 720P 2000； H265可再降低500
    video_item["min_bps"]=min_bps               #视频传输弱网码率，弱网下能接受的最小码率，建议1080P 1500， 720P 1000； H265可再降低300
    video_item["codec"]=codec    #编码方式 0 H264  1 H265
    video_item["protocol"]="outside"
    return video_item

#外部输入编码流配置生成，external encoded video input video stream
def outenc_video_item(codec=0):
    video_item=OrderedDict()
    video_item["fps"]=30
    video_item["width"]=1920
    video_item["height"]=1080
    video_item["codec"]=codec    #编码方式 0 H264  1 H265
    video_item["protocol"]="out_enc"
    return video_item

#网络相机rtsp编码透传视频流配置生成，rtsp encoded stream input with nodecoding
def rtsp_enc_video_item(fps, width, height, rtsp_url, codec=0):
    video_item=OrderedDict()
    video_item["fps"]=fps
    video_item["width"]=width
    video_item["height"]=height
    video_item["protocol"]="rtsp_enc"
    video_item["codec"]=codec
    cameras=[]
    camera_item={}
    camera_item["protocol"]=1
    camera_item["url"]=rtsp_url
    camera_item["width"]=width
    camera_item["height"]=height
    camera_item["fps"]=fps
    cameras.append(camera_item)
    video_item["cameras"]=cameras
    return video_item

#网络相机rtsp转码视频流配置生成， rtsp/file url stream input with decoding
def normal_url_video_item(fps,width, height, encode_width, encode_height, bps, min_bps, url, codec=0):
    video_item=OrderedDict()
    video_item["fps"]=fps
    video_item["width"]=width
    video_item["height"]=height
    video_item["encode_width"]=encode_width
    video_item["encode_height"]=encode_height
    video_item["bps"]=bps
    video_item["min_bps"]=min_bps
    video_item["codec"]=codec
    video_item["protocol"]="normal"
    cameras=[]
    camera_item=OrderedDict()
    camera_item["protocol"]=1
    camera_item["url"]= url
    camera_item["width"]=width
    camera_item["height"]=height
    camera_item["fps"]=fps
    cameras.append(camera_item)
    video_item["cameras"]=cameras
    return video_item



gen_config()
