#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <thread>
#include "trro_field.h"
#include <map>
#include <mutex>
#include <condition_variable>
#include <chrono>

static std::map<int, std::string> g_Record_Files;
std::mutex retry_mtx; 
std::condition_variable retry_cv; 
#define MAX_RETRY 3 // 公有云lincense超时最大重连次数

static inline long long GetTime2() {

	auto now = std::chrono::system_clock::now();
	auto duration = now.time_since_epoch();
	auto millis = std::chrono::duration_cast<std::chrono::milliseconds>(duration).count();

	return millis;
}

void onControlData(void * context, const char* controlid, const char* msg, int len, int cmd) {
	
	printf("received controller msg: %s\n",msg);
	std::string out(msg);
	auto pos = out.find_last_of(':');
    if (pos != std::string::npos) {
        std::string old = out.substr(pos + 1, out.length());
        long long oldtime = std::atoll(old.c_str());
        long long newtime = GetTime2();
    } 

	TRRO_sendControlData(msg, len);
}

void OnConnectState(void* context, int stream_id, int state) {
	printf("stream_id: %d, state: %d\n", stream_id, state);
}

void externalOutYuvBuffer(const char* filename, int id) {
	int data_size = 1280*720*3/2;
	int data_width = 1280;
	int data_height = 720;
	FILE *yuv_file = fopen(filename, "rb+");
	char *data = (char*)malloc(data_size);
	memset(data, 0, data_size);
	TRRO_setChineseFontAndSize("./simhei.ttf", 40, "你好");
	while(true) {
		if(yuv_file == nullptr) {
			return;
		}
		
		int size = fread(data, 1, data_size, yuv_file);
		if (size < 1280) {
			fseek(yuv_file, 0 , SEEK_SET);
			fread(data, 1, data_size, yuv_file);
		}
		TRRO_TextFormat format;
		format.text = "你好";
		format.point_x = data_width / 2;
		format.point_y = data_height / 2;
		format.text_color[0] = 0;
		format.text_color[1] = 0;
		format.text_color[2] = 255;
		format.text_color[3] = 0;
		format.text_border_color[0] = 0;
		format.text_border_color[1] = 0;
		format.text_border_color[2] = 255;
		format.text_border_color[3] = 0;
		format.text_border_size = 1;
		TRRO_externalVideoDataWithText(id, data, data_width, data_height, 0, 0, &format);
		usleep(33 * 1000);
	}
	if (yuv_file) {
		fclose(yuv_file);
	}
	free(data);
}


int main() {
	TRRO_registerSignalStateCallback(nullptr, [](void *context, SignalState state) {
		if(state == kTrroReady) {
			printf("TRRO_init >> init success \n");
			TRRO_registerControlDataCallback(nullptr, onControlData);
			TRRO_registerOnState(nullptr, OnConnectState);
			TRRO_registerOnErrorEvent(nullptr, [](void* context, int error_code, const char* error_msg) {
				printf("error_code %d, error_msg %s\n", error_code, error_msg);
			});

			TRRO_registerVideoCaptureCallback(nullptr, [](void *context, const char* data, int width, int height, int type, int stream_id) {});
			TRRO_registerLatencyCallback(nullptr, [](void *context, int stream_id, int vcct){
				printf("latency stream id %d, vcct %d\n", stream_id, vcct);
			});

			TRRO_registerAudioMediaState(nullptr, [](void* context, int stream_id, int fps, int bps, int rtt, long long lost, long long packets_send, int stun) {
				printf("audio stream %d, fps %d, bps %d, rtt %d, lost %lld, packets_send %lld, stun %d\n", stream_id, fps, bps, rtt, lost, packets_send, stun);
			});

			TRRO_registerMediaState(nullptr, [](void* context, int stream_id, int fps, int bps, int rtt, long long lost, long long packets_send, int stun) {
				printf("stream %d, fps %d, bps %d, rtt %d, lost %lld, packets_send %lld, stun %d\n", stream_id, fps, bps, rtt, lost, packets_send, stun);
			});
			
			TRRO_registerOperationPermissionRequest(nullptr, [](void* context, const char* remote_devid, int permission) {
				printf("remote devid %s permission %d\n", remote_devid, permission);
			});
			retry_cv.notify_one(); //mqtt连接成功，通知trro_start 
		} else if(state == kTrroAuthFailed) {
			printf("device_id or password is incorrect\n");
		} else if(state == kTrroKickout) {
			printf("mqtt kickout, stop sdk\n");
		} else if(state == kTrroLost) {
			printf("disconnected , connecting...  \n");
		} else if(state == kTrroReup) {
			printf("reconnect sucess\n");
		}
	});

	// 最后一个参数改为0 恢复会旧版阻塞模式 可以启用 后续注释代码并注释上面的回调TRRO_registerSignalStateCallback
	int ret = TRRO_initGwPathWithLicense("./config.json", "./license.txt", -1);
	if(TRRO_SUCCED != ret) {
		const char* init_msg = getErrorMsg(ret);
		printf("Trro_init >> %s, ret: %d\n", init_msg, ret);
	}

 	{
		// TRRO_start及公有云license验证超时重连
        std::unique_lock<std::mutex> lock(retry_mtx);
		retry_cv.wait(lock);
		int retryCount = 0; 
	
		while (retryCount < MAX_RETRY) {
			int ret = TRRO_start();
			if(ret == TRRO_SUCCED) {
				const char* start_msg = getErrorMsg(ret);
				printf("TRRO_start >> %s\n", start_msg);
				break; 
			} else if(ret == -TRRO_INIT_PUBLIC_LICENSE_CHECK_TIMEOUT) {
				retryCount++; 
				printf("TRRO_start >> License check timeout, reconnecting count: %d\n", retryCount);
				sleep(1);
			} else {
				const char* start_msg = getErrorMsg(ret);
				printf("TRRO_start >> %s, ret: %d\n", start_msg, ret);
				break;
			}
		}
		if(retryCount == MAX_RETRY) {
			printf("TRRO_start >> License check timeout, reconnecting failed\n");
		}
        
    }

	const char* ver = TRRO_getSdkVersion();
	printf("sdk version is %s\n", ver);
	// std::thread thread1([] {
	// 	sleep(10);
	// 	int i = 0;
	// 	int ret = TRRO_testNetworkQuality(&i, 1, 0);
	// 	printf("TRRO_testNetworkQuality ret = %d\n", ret);

	// 	sleep(5);
	// 	int ii[] = { 0, 1, 2};
	// 	ret = TRRO_testNetworkQuality(ii, 2, 40);
	// 	printf("TRRO_testNetworkQuality ret = %d\n", ret);

	// 	sleep(5);
	// 	ret = TRRO_testNetworkQuality(ii, 2, 6);
	// 	printf("TRRO_testNetworkQuality ret = %d\n", ret);
	// });

	/* 外部输入YUV使用如下接口*/
    // std::thread t1(externalOutYuvBuffer, "./720p_30.yuv", 0);
    // t1.detach();

	// unsigned long long capture_id = 0;
	// TRRO_startVideoCapture(nullptr, "/dev/video8", (VideoCaptureProtocol)1, (TrroColor)4, 1920, 1080, 30, 
	// 	[](void *context, unsigned long long capture_id, const char* data, int length, int width, int height, TrroColor color_format){
	// 		TRRO_externalVideoData(0, data, width, height, color_format);
	// 	}, 
	// &capture_id);

    while(true){
        sleep(30000);
    }
	return 0;

}


